import { Request, Response } from "express";
import { Route, RouteMessages } from "server";
import { nodeManager } from "..";

const IGNORED_PATHS = ["/favicon.ico"];

export default class ProxyRoute extends Route {
  constructor() {
    super({ path: "/" });
  }

  async handle(req: Request, res: Response) {
    const url = req.url.substring(1);
    if (!url) {
      res.status(400).json(RouteMessages.badRequest("No URL provided"));
      return;
    }
    if (IGNORED_PATHS.includes(url)) {
      res.status(404).send("Not found");
      return;
    }

    const node = nodeManager.getRandomNode();
    if (!node) {
      res
        .status(500)
        .json(RouteMessages.internalServerError("No nodes available"));
      return;
    }

    try {
      const before = Date.now();
      const response = await node.fetch(url);
      const data = response.data;

      if (response.status === 500) {
        res.status(500).json(RouteMessages.internalServerError(data));
        return;
      }

      // Log the request
      console.log(
        `[node: ${response.headers["x-proxy-node"]}] ${url} - ${
          response.status
        } (${Date.now() - before}ms)`
      );

      // Return the JSON response
      res.status(response.status).set(response.headers).json(data);
    } catch (ex: any) {
      res.status(500).json(RouteMessages.internalServerError(ex.message || ex));
    }
  }
}
